import java.util.Objects;

public class Account {

    private String owner;
    private int amount;

    public Account(String fullname) {
        this.owner = fullname;
    }
    public Account(String fullname, int amount) {
        this(fullname);
        this.deposit(amount);
    }

    public int amount() {
        return this.amount;
    }

    public boolean canWithdraw(int value) {
        return amount() >= value;
    }

    public String fullname() {
        return this.owner;
    }

    private static void checkPositiveOrThrow(int i) {
        if (i < 0) {
            throw new RuntimeException("Amount should be positive");
        }
    }
    public void deposit(int amountToPlace) {
        Account.checkPositiveOrThrow(amountToPlace);
        this.amount += amountToPlace;
    }

    public void withdraw(int amountToWithdraw) {
        Account.checkPositiveOrThrow(amountToWithdraw);
        if ( this.amount - amountToWithdraw <= -2000 ) {
            throw new RuntimeException("withdraw unpossible");
        }
        this.amount -= amountToWithdraw;
    }

    public void transferTo(Account destination, int amountToTransfer) {
        this.withdraw(amountToTransfer);
        destination.deposit(amountToTransfer);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || o.getClass() != this.getClass()) {
            return false;
        }
        Account that = (Account)o;
        return that.owner.equals(this.owner) && that.amount == this.amount;
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.owner, this.amount);
    }


}