public class Container {
   private final int totalCapacity;
   private int currentValue;

   public Container(int capacity) {
      ensureCapacity(capacity);
      this.totalCapacity = capacity;
   }

   private void ensureCapacity(int capacity) {
      if (capacity < 0) throw new IllegalStateException("Capacity should be positive or null");
   }

   public void fillToTheMax() {
      this.currentValue = totalCapacity;
   }

   /* Try to add `value` to the container
    * @return the quantity that could have been added
    */
   public int fillWith(int value) {
      ensureCapacity(value);
      final int fillable = fillableAccordingTo(value);
      this.currentValue += fillable;
      return fillable;
   }

   public int remaining() {
      return this.totalCapacity - this.currentValue;
   }

   public int currentValue() {
      return this.currentValue;
   }

   public void flush() {
      this.currentValue = 0;
   }

   public int quantity() {
        return this.currentValue;
    }

   public void remove(int value) {
        if (this.currentValue - value >= 0) this.currentValue -= value;
    }

   public boolean isFull() {
      return this.currentValue == this.totalCapacity;
   }

   public void fillTo(Container dest1, Container... dests) {
      int taken = dest1.fillWith(this.currentValue);
      this.currentValue -= taken;
      for(Container d: dests) {
         taken = d.fillWith(this.currentValue);
         this.currentValue -= taken;
      }
   }

   public int fillableAccordingTo(int value) {
      return ContainerHelper.canTransfer(value, remaining());
   }

   public static Container withCapacity(int capacity) {
      return new Container(capacity);
   }
}