import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Month;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import java.io.Serializable;
import java.util.List;

public class Individu implements Serializable {
    // Enum pour les lieux de naissance
    public enum LieuxDeNaissance {
        TOKYO, PARIS, NAIROBI, LOS_ANGELES, BAMAKO, MOSCOU, CARACAS, TBILISSI
    }

    // Enum pour les hobbies
    public enum Hobbies {
        JEUX_VIDEO, SPORT, RANDONNÉE, SKI, CINÉMA, CUISINE, LECTURE
    }

    // Attributs privés
    private final int taille;
    private final LocalDateTime dateDeNaissance;
    private final LieuxDeNaissance lieuDeNaissance;
    private final int poids;
    private final List<Hobbies> hobbies;

    // Constructeur
    public Individu(int taille, LocalDateTime dateDeNaissance, LieuxDeNaissance lieuDeNaissance, int poids, List<Hobbies> hobbies) {
        if (taille < 150 || taille > 190) {
            throw new IllegalArgumentException("La taille doit être comprise entre 150cm et 190cm.");
        }
        if (poids < 60 || poids > 100) {
            throw new IllegalArgumentException("Le poids doit être compris entre 60kg et 100kg.");
        }
        this.taille = taille;
        this.dateDeNaissance = dateDeNaissance;
        this.lieuDeNaissance = lieuDeNaissance;
        this.poids = poids;
        this.hobbies = hobbies;
    }

    // Getters
    public int getTaille() {
        return taille;
    }

    public LocalDateTime getDateDeNaissance() {
        return dateDeNaissance;
    }

    public LieuxDeNaissance getLieuDeNaissance() {
        return lieuDeNaissance;
    }

    public int getPoids() {
        return poids;
    }

    public List<Hobbies> getHobbies() {
        return hobbies;
    }

    // Affichage lisible
    @Override
    public String toString() {
        return String.format("Né le %s à %s, mesure %dcm, pèse %dkg, aime %s",
                dateDeNaissance.toLocalDate(),
                lieuDeNaissance,
                taille,
                poids,
                hobbies.toString().replace("[", "").replace("]", ""));
    }

    // Affichage formaté pour la sauvegarde dans un fichier
    public String toCSV() {
        return String.format(
            "%s,%s,%d,%d,%s",
            dateDeNaissance.toLocalDate(),
            lieuDeNaissance,
            taille,
            poids,
            String.join(";", hobbies.stream().map(Enum::name).toList())
        );
    }

    // Création d'un individu à partir d'une ligne CSV
    public static Individu fromCSV(String csvLine) {
        String[] parts = csvLine.split(",");
        LocalDateTime dateDeNaissance = LocalDateTime.of(
            LocalDate.parse(parts[0]),
            LocalTime.of(0, 0)
        );
        LieuxDeNaissance lieuDeNaissance = LieuxDeNaissance.valueOf(parts[1]);
        int taille = Integer.parseInt(parts[2]);
        int poids = Integer.parseInt(parts[3]);
        List<Hobbies> hobbies = Arrays.stream(parts[4].split(";"))
                .map(Hobbies::valueOf)
                .toList();
        return new Individu(taille, dateDeNaissance, lieuDeNaissance, poids, hobbies);
    }
}
