import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Month;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Supplier;
import java.io.File;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;

public class Test2 {
    // Méthode surchargée pour contrôler le type de liste (ArrayList)
    public static List<Individu> createIndividusArrayList(int n) {
        return new ArrayList<>(IndividuUtils.createIndividus(n));
    }

    // Méthode surchargée pour contrôler le type de liste (LinkedList)
    public static List<Individu> createIndividusLinkedList(int n) {
        return new LinkedList<>(IndividuUtils.createIndividus(n));
    }

    // Méthode surchargée pour contrôler la source avec Supplier
    public static List<Individu> createIndividus(Supplier<Individu> supplier, int n) {
        List<Individu> individus = new ArrayList<>();
        for (int i = 0; i < n; i++) {
            individus.add(supplier.get());
        }
        return individus;
    }

    // Méthode test pour afficher la liste des individus
    public static void test(List<Individu> individus) {
        individus.forEach(individu -> {
            System.out.println(individu);
            System.out.println("Hobbies: " + individu.getHobbies());
        });
    }

    // Méthode surchargée pour effectuer l'action de "sortie"
    public static void testWithConsumer(List<Individu> individus, Consumer<Individu> consumer) {
        individus.forEach(consumer);
    }

    // Consumer pour écrire dans un fichier
    public static Consumer<Individu> writeToFileConsumer(String fileName) {
        return individu -> {
            try (FileWriter writer = new FileWriter(fileName, true)) {
                writer.write(individu.toCSV() + "\n");
            } catch (IOException e) {
                e.printStackTrace();
            }
        };
    }

    // Supplier pour lire depuis un fichier
    public static Supplier<Individu> readFromFileSupplier(String fileName) {
        return new Supplier<>() {
            private BufferedReader reader;
            {
                try {
                    reader = new BufferedReader(new FileReader(new File(fileName)));
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public Individu get() {
                try {
                    if (reader != null) {
                        String line = reader.readLine();
                        if (line != null) {
                            return Individu.fromCSV(line);
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
                return null;
            }
        };
    }

    // Main pour tester les méthodes
    public static void main(String[] args) {
        System.out.println("Créer des individus avec ArrayList :");
        List<Individu> individusArrayList = createIndividusArrayList(5);
        test(individusArrayList);

        System.out.println("\nCréer des individus avec LinkedList :");
        List<Individu> individusLinkedList = createIndividusLinkedList(5);
        test(individusLinkedList);

        System.out.println("\nCréer des individus avec Supplier :");
        Supplier<Individu> singleSupplier = () -> new Individu(
            170,
            LocalDateTime.of(1995, Month.JANUARY, 1, 0, 0),
            Individu.LieuxDeNaissance.PARIS,
            70,
            List.of(Individu.Hobbies.CINÉMA, Individu.Hobbies.SPORT)
        );

        // ou 
        // Supplier<Individu> singleSupplier = () -> IndividuUtils.createFixedIndividu();

        List<Individu> individusFromSupplier = createIndividus(singleSupplier, 3);
        test(individusFromSupplier);

        System.out.println("\nTester avec un Consumer :");
        String fileName = "individus.csv";
        Consumer<Individu> fileWriterConsumer = writeToFileConsumer(fileName);
        testWithConsumer(individusArrayList, fileWriterConsumer);

        System.out.println("\nLire depuis le fichier avec Supplier :");
        Supplier<Individu> fileReaderSupplier = readFromFileSupplier(fileName);
        Individu individuFromFile;
        while ((individuFromFile = fileReaderSupplier.get()) != null) {
            System.out.println("Lu depuis le fichier : " + individuFromFile);
        }
    }
}
